from slai.types import ModelTypes
from slai.base_handler import BaseModelHandler
from slai.login import Login
from slai.model import Model

__version__ = "0.1.86"


# most used slai actions go here
model = Model
types = ModelTypes
login = Login
BaseModelHandler = BaseModelHandler

__all__ = [
    "__version__",
    "model",
    "types",
    "base_handler",
    "login",
]



import requests

from requests.auth import HTTPBasicAuth
from importlib import import_module

from slai.config import get_api_base_urls
from slai.modules.parameters import from_config
from slai.modules.runtime import detect_credentials

REQUESTS_TIMEOUT = 15


def get_cli_client(client_id=None, client_secret=None):
    import_path = from_config(
        "CLI_CLIENT",
        "slai.clients.cli.SlaiCliClient",
    )
    class_ = import_path.split(".")[-1]
    path = ".".join(import_path.split(".")[:-1])
    return getattr(import_module(path), class_)(
        client_id=client_id, client_secret=client_secret
    )


class SlaiCliClient:
    BACKEND_BASE_URL, _ = get_api_base_urls()

    def __init__(
        self,
        client_id=None,
        client_secret=None,
        user_agent_header="SlaiCli smartshare/0.1.0",
    ):

        if client_id is None or client_secret is None:
            credentials = detect_credentials()
            self.client_id = credentials["client_id"]
            self.client_secret = credentials["client_secret"]
        else:
            self.client_id = client_id
            self.client_secret = client_secret

        self.user_agent_header = user_agent_header

    def retrieve_sandbox(self, *, uri):
        body = {"action": "retrieve", "uri": uri}
        body = {k: v for k, v in body.items() if v is not None}

        res = requests.post(
            f"{self.BACKEND_BASE_URL}/sandbox/sandbox",
            auth=HTTPBasicAuth(self.client_id, self.client_secret),
            headers={
                "User-Agent": self.user_agent_header,
            },
            json=body,
            timeout=REQUESTS_TIMEOUT,
        )
        res.raise_for_status()
        return res.json()

    def list_model_artifacts(self, *, model_version_id):
        body = {
            "action": "list",
            "model_version_id": model_version_id,
        }

        res = requests.post(
            f"{self.BACKEND_BASE_URL}/cli/model-artifact",
            auth=HTTPBasicAuth(self.client_id, self.client_secret),
            headers={
                "User-Agent": self.user_agent_header,
            },
            json=body,
            timeout=REQUESTS_TIMEOUT,
        )
        res.raise_for_status()
        return res.json()

    def retrieve_model_artifact(self, *, model_version_id, model_artifact_id):
        body = {
            "action": "retrieve",
            "model_version_id": model_version_id,
            "model_artifact_id": model_artifact_id,
        }

        body = {k: v for k, v in body.items() if v is not None}

        res = requests.post(
            f"{self.BACKEND_BASE_URL}/cli/model-artifact",
            auth=HTTPBasicAuth(self.client_id, self.client_secret),
            headers={
                "User-Agent": self.user_agent_header,
            },
            json=body,
            timeout=REQUESTS_TIMEOUT,
        )
        res.raise_for_status()
        return res.json()

    def get_user(self):
        body = {"action": "retrieve"}

        res = requests.post(
            f"{self.BACKEND_BASE_URL}/cli/user",
            auth=HTTPBasicAuth(self.client_id, self.client_secret),
            headers={
                "User-Agent": self.user_agent_header,
            },
            json=body,
            timeout=REQUESTS_TIMEOUT,
        )
        res.raise_for_status()
        return res.json()

    def get_cli_version(self):
        body = {}
        res = requests.post(
            f"{self.BACKEND_BASE_URL}/cli/cli-version",
            headers={
                "User-Agent": self.user_agent_header,
            },
            json=body,
            timeout=REQUESTS_TIMEOUT,
        )
        res.raise_for_status()
        return res.json()



import os
import sys
import yaml

from pathlib import Path
from slai.exceptions import NoCredentialsFound

LOCAL_CREDENTIALS_PATHS = {"global": f"{Path.home()}/.slai/credentials.yml"}


def detect_credentials(*, profile_name="default"):
    credentials = getattr(sys.modules["slai"], "credentials", None)

    if credentials:
        return credentials

    if os.path.exists(LOCAL_CREDENTIALS_PATHS["global"]):
        credentials = _load_credentials(
            path=LOCAL_CREDENTIALS_PATHS["global"], credentials_type="global"
        )
        credentials = credentials.get(profile_name)

        if not credentials:
            raise NoCredentialsFound("no_credentials_found")

    else:
        raise NoCredentialsFound("no_credentials_found")

    return credentials


def _load_credentials(*, path, credentials_type="global"):
    credentials = {}

    with open(path, "r") as f_in:
        try:
            credentials = yaml.safe_load(f_in)
        except yaml.YAMLError:
            raise NoCredentialsFound("slai_invalid_config")

    if credentials_type == "global":
        return credentials

    return credentials



import requests
import copy

from functools import lru_cache
from requests.auth import HTTPBasicAuth
from importlib import import_module
from slai.modules.parameters import from_config
from slai.config import get_api_base_urls
from slai.modules.runtime import detect_credentials
from slai.clients.model import ModelClient, get_model_client
from slai.types import ModelTypes, InvalidPayloadException, InvalidTypeException

REQUESTS_TIMEOUT = 180


def get_inference_client(*, model_uri):
    import_path = from_config(
        "MODEL_INFERENCE_CLIENT",
        "slai.clients.inference.ModelInferenceClient",
    )
    class_ = import_path.split(".")[-1]
    path = ".".join(import_path.split(".")[:-1])

    return getattr(import_module(path), class_)(
        model_uri=model_uri,
    )


@lru_cache(maxsize=64)
def _get_model_info(
    base_url,
    client_id,
    client_secret,
    sandbox_id,
):
    body = {"sandbox_id": sandbox_id}

    res = requests.post(
        f"{base_url}/model/info",
        auth=HTTPBasicAuth(client_id, client_secret),
        json=body,
        timeout=REQUESTS_TIMEOUT,
    )
    res.raise_for_status()
    model_info = res.json()

    input_schema = ModelTypes.load_schema(model_info["input_schema"])
    output_schema = ModelTypes.load_schema(model_info["output_schema"])
    inference_url = model_info["inference_url"]

    return input_schema, output_schema, inference_url


class ModelInferenceClient:
    BACKEND_BASE_URL, _ = get_api_base_urls()

    def __init__(self, *, model_uri):
        credentials = detect_credentials()

        self.client_id = credentials["client_id"]
        self.client_secret = credentials["client_secret"]
        self.model_uri = model_uri

        self._load_model()

    def _load_model(self):
        self.model_client: ModelClient = get_model_client(
            model_uri=self.model_uri,
        )
        self.sandbox = self.model_client.get_sandbox()

    def call(self, payload):
        input_schema, output_schema, inference_url = _get_model_info(
            self.BACKEND_BASE_URL,
            self.client_id,
            self.client_secret,
            self.sandbox["short_id"],
        )

        try:
            serialized_payload = ModelTypes.serialize(payload, input_schema)
        except (InvalidTypeException, InvalidPayloadException) as e:
            print(e, e.errors)
            return None

        res = requests.post(
            f"{inference_url}",
            auth=HTTPBasicAuth(self.client_id, self.client_secret),
            json=serialized_payload,
            timeout=REQUESTS_TIMEOUT,
        )
        res.raise_for_status()
        response_data = res.json()

        errors = response_data["result"].get("errors", [])
        if errors:
            print("An unhandled exception occurred in your handler:\n ")

            for e in errors:
                print(e)

            return None

        try:
            result = ModelTypes.deserialize(response_data["result"], output_schema)
        except (InvalidTypeException, InvalidPayloadException) as e:
            print(e, e.errors)
            return None

        return result

    def info(self):
        return copy.copy(
            _get_model_info(
                self.BACKEND_BASE_URL,
                self.client_id,
                self.client_secret,
                self.sandbox["short_id"],
            )
        )



from slai.modules.parameters import from_config


def get_api_base_urls():
    stage = from_config(
        key="STAGE",
        default="production",
    )

    if stage == "local":
        backend_base_url = from_config(
            key="SLAI_BASE_URL",
            default="http://localhost:8080",
        )
        model_base_url = "models.slai.local"
    elif stage == "development":
        backend_base_url = "https://api.eng-dev.slai.io"
        model_base_url = "https://models.eng-dev.slai.io"
    elif stage == "staging":
        backend_base_url = "https://api.eng-stage.slai.io"
        model_base_url = "https://models.eng-staging.slai.io"
    else:
        backend_base_url = from_config(
            # the previous BASE_URL conflicted with HEX Notebook env
            # Forcing it to default to api.slai.io
            key="SLAI_BASE_URL",
            default=f"https://api.slai.io",
        )
        model_base_url = "https://models.slai.io"

    return backend_base_url, model_base_url



from slai.clients.inference import get_inference_client


class Model:
    def __init__(self, model_uri):
        self.model_uri = model_uri
        self.inference_client = get_inference_client(
            model_uri=self.model_uri,
        )

    def __call__(self, **inputs):
        return self.inference_client.call(payload=inputs)

    def call(self, **inputs):
        return self.inference_client.call(payload=inputs)

    def info(self):
        return self.inference_client.info()



from slai.exceptions import NoCredentialsFound
import requests

from requests.auth import HTTPBasicAuth
from importlib import import_module

from slai.modules.parameters import from_config
from slai.config import get_api_base_urls
from slai.modules.runtime import detect_credentials


REQUESTS_TIMEOUT = 15


def get_identity_client(*, client_id=None, client_secret=None, key_type=None):
    import_path = from_config(
        "IDENTITY_CLIENT",
        "slai.clients.identity.IdentityClient",
    )
    class_ = import_path.split(".")[-1]
    path = ".".join(import_path.split(".")[:-1])
    return getattr(import_module(path), class_)(
        client_id=client_id, client_secret=client_secret, key_type=key_type
    )


class IdentityClient:
    BACKEND_BASE_URL, _ = get_api_base_urls()

    def __init__(self, client_id=None, client_secret=None, key_type=None):
        self.credentials_loaded = False
        self.key_type = key_type

        if not client_id or not client_secret:
            try:
                self.credentials = detect_credentials()
                self.client_id = self.credentials["client_id"]
                self.client_secret = self.credentials["client_secret"]
                self.credentials_loaded = True
            except NoCredentialsFound:
                pass

        else:
            self.client_id = client_id
            self.client_secret = client_secret

            self.credentials_loaded = True

    def get_user(self):
        if not self.credentials_loaded:
            raise NoCredentialsFound("no_credentials_loaded")

        body = {"action": "retrieve"}

        res = requests.post(
            f"{self.BACKEND_BASE_URL}/cli/user",
            auth=HTTPBasicAuth(self.client_id, self.client_secret),
            headers={},
            json=body,
            timeout=REQUESTS_TIMEOUT,
        )
        res.raise_for_status()
        return res.json()

    def validate_credentials(self):
        if not self.credentials_loaded:
            raise NoCredentialsFound("no_credentials_loaded")

        body = {}
        if self.key_type is not None:
            body["key_type"] = self.key_type

        res = requests.post(
            f"{self.BACKEND_BASE_URL}/cli/validate-credentials",
            auth=HTTPBasicAuth(self.client_id, self.client_secret),
            headers={},
            json=body,
            timeout=REQUESTS_TIMEOUT,
        )
        res.raise_for_status()
        return res.json()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['slai', 'slai.clients', 'slai.modules']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'python-dotenv>=0.15.0,<0.16.0',
 'requests>=2.26.0,<3.0.0',
 'urllib3>=1.26.7,<2.0.0']

setup_kwargs = {
    'name': 'slai',
    'version': '0.1.87',
    'description': '',
    'long_description': 'None',
    'author': 'slai',
    'author_email': 'luke@slai.io',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



from slai.clients.cli import SlaiCliClient, get_cli_client
from slai.modules.parameters import from_config
from importlib import import_module
from functools import lru_cache


def get_model_client(*, model_uri):
    import_path = from_config(
        "MODEL_CLIENT",
        "slai.clients.model.ModelClient",
    )
    class_ = import_path.split(".")[-1]
    path = ".".join(import_path.split(".")[:-1])
    return getattr(import_module(path), class_)(
        model_uri=model_uri,
    )


class ModelClient:
    def __init__(
        self,
        *,
        model_uri: str,
    ):
        self.model_uri = model_uri
        self.cli_client: SlaiCliClient = get_cli_client()
        self.sandbox = self.get_sandbox()

    @lru_cache(maxsize=64)
    def get_sandbox(self):
        sandbox_data = self.cli_client.retrieve_sandbox(
            uri=self.model_uri,
        )
        return sandbox_data

    def get_latest_model_artifact(self):
        model_artifact = self.cli_client.retrieve_model_artifact(
            model_version_id=self.sandbox["model_version_id"],
            model_artifact_id=None,
        )
        return model_artifact

    def list_model_artifacts(self):
        model_artifacts = self.cli_client.retrieve_model_artifact()
        return model_artifacts
